/* Foldroid - Mail Folder Selection Tool
 * Copyright (C) 2010 Stefan Westerfeld
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <unistd.h>
#include "fddfolderthread.h"
#include <pthread.h>
#include <stdio.h>
#include <string.h>

using std::string;

using namespace Foldroid;

FolderThread *instance = NULL;
pthread_t thread;

void*
thread_start (void *arg)
{
  instance->run();
}

void
FolderThread::start()
{
  instance = new FolderThread();
  pthread_create (&thread, NULL, thread_start, instance);
}

void
FolderThread::stop()
{
  Command cmd;
  cmd.command = COMMAND_STOP;
  pthread_mutex_lock (&instance->m_folder_thread_mutex);
  instance->m_command_queue.push_back (cmd);
  pthread_mutex_unlock (&instance->m_folder_thread_mutex);

  void *result;
  pthread_join (thread, &result);
  delete instance;
}

void
FolderThread::request_scan (const string& filename)
{
  Command cmd;
  cmd.command = COMMAND_SCAN;
  cmd.filename = filename;

  pthread_mutex_lock (&instance->m_folder_thread_mutex);
  instance->m_command_queue.push_back (cmd);
  pthread_mutex_unlock (&instance->m_folder_thread_mutex);
}

FolderThread*
FolderThread::the()
{
  return instance;
}

void
FolderThread::run()
{
  pthread_mutex_init (&m_folder_thread_mutex, NULL);
  for (;;)
    {
      pthread_mutex_lock (&m_folder_thread_mutex);
      while (m_command_queue.size())
        {
          Command cmd = m_command_queue[0];
          m_command_queue.erase (m_command_queue.begin());
          switch (cmd.command)
            {
              case COMMAND_STOP:
                return;
              case COMMAND_SCAN:
                {
                  pthread_mutex_unlock (&m_folder_thread_mutex);
                  scan (cmd.filename);
                  pthread_mutex_lock (&m_folder_thread_mutex);
                }
                break;
            }
        }
      pthread_mutex_unlock (&m_folder_thread_mutex);
      sleep (1);
    }
}

void
FolderThread::scan (const string& filename)
{
  int nFlag = 0;
  int messages = 0;

  FILE *folder = fopen (filename.c_str(), "r");
  if (folder)
    {
      enum { inHeader, inBody } state = inHeader;
      char buffer[1024];
      string status;

      while (fgets (buffer, 1024, folder))
        {
          const char *first = strtok(buffer, " \n");
          if (first)
            {
              const char *second = strtok(NULL, "\n");
              if (second && buffer[0] != ' ')
                {
                  if (strcmp (first, "From") == 0)
                    {
                      state = inHeader;
                      status = "";
                    }
                  else if (state == inHeader && (strcmp (first, "Status:") == 0))
                    {
                      status = second;
                    }
                }
            }
          else
            {
              if (state == inHeader)
                {
                  int old = 0, read = 0;

                  string::const_iterator si;
                  for (si = status.begin(); si != status.end(); si++)
                    {
                      if (*si == 'O')
                        old = 1;
                      if (*si == 'R')
                        read = 1;
                    }

                  if (!old)
                    nFlag++;
                  messages++;
                }
              state = inBody;
            }
        }
      fclose (folder);
    }

  CommandResult result;
  result.command = COMMAND_SCAN;
  result.new_flag = nFlag;
  result.messages = messages;
  result.filename = filename;

  pthread_mutex_lock (&m_folder_thread_mutex);
  m_result_queue.push_back (result);
  pthread_mutex_unlock (&m_folder_thread_mutex);
}

bool
FolderThread::get_scan_result (CommandResult& result)
{
  bool have_result = false;

  pthread_mutex_lock (&m_folder_thread_mutex);
  if (!m_result_queue.empty())
    {
      result = m_result_queue[0];
      m_result_queue.erase (m_result_queue.begin());
      have_result = true;
    }
  pthread_mutex_unlock (&m_folder_thread_mutex);

  return have_result;
}
